#!/usr/bin/bash

#set -euo pipefail

if [ $# -lt 2 ]; then
    echo "Usage: $0 <channel> <name>"
    exit 1
fi

# ------------------------------------------------

CHANNEL=$1
MEMBER=$2
PASSWORD=$3
MAXCHAR=29 # max length of keyboard input line
inline="" # initialise line buffer value as empty
INBUFFER=""; # create line transfer buffer
TAILLINES=$4

# Set trap to kill background child tail process when Ctl+C is pressed
trap 'kill $tailpidout; exit' INT


# Display chat history 20 last lines and current keyboard input
# ----------------------------------------------------------------

FGC=$(( 16#$(\echo "$MEMBER" | /usr/bin/md5sum | /usr/bin/head -c2) ))  # Arbitrary number in [0:255]
BGC=$(( ("$FGC" + 128) % 255 ))  # Most of the time, high enough contrast


tail -f -n $TAILLINES ./chat/"$CHANNEL" 2> /dev/null | while read b; do echo "$b" \
   | openssl enc -aes128 -pbkdf2 -A -a -d -k $PASSWORD | sed -e $'s/^/\033[A\33[2K\r/' \
   | sed -e "s/$/\n~~~~~ $(printf '\e[38;5;%dm\e[48;5;%dm%-9s\e[0m %s\e[0m\n' $FGC $BGC $MEMBER)~~~~~~~~~~~~~>---------------/" ; done &
tailpidout=$!

# Running the chat keyboard input recordings
# ----------------------------------------------------

sleep 0.3;
while true
do
# Reading keyboard input without waiting for enter and keeping the line in fifo buffer (file)
sleep 0.1 # delay to be sure keyboard input is printed on the screen below line buffer (file) tailed line
INBUFFER=$inline # clearing linebuffer file (or entering lastchar)
echo -ne "\r\033[K$inline" # print line to screen
  while :; do # character input from keyboard reading cycle
    read -t 0.1 -s -N 1 char; # read keyboard input character for 1 second
    rdst=$? # get read status - if no input, status is greater than 128
    if [[ "$char" == $'\x0a' ]] || [[ ${#inline} -ge $MAXCHAR ]]; then # enter key is pressed or line lenght is maximum
        if [[ "$char" == $'\x0a' ]]; # pressed enter
            then
              inline=""; # reset input line if ended by enter
            else # save and print last character if max length exceeded
               echo -ne "\r\033[K$inline" # print line to screen
                 inline=$char;
        fi
        if [[ ! -z $INBUFFER ]]; then break; fi # exit if line is not empty, otherwise wait for input
    else
        if [[ "$char" == $'\x7f' ]]; then # backspace key is pressed.
            if [[ ! -z $inline ]]; then inline=${inline::-1}; fi # delete last character of the line if it is not empty
            echo -ne "\r\033[K$inline" # print line to screen
            INBUFFER=$inline # write line to fifo buffer
        else
           echo -ne "\r\033[K$inline" # print line to screen
           if [[ "$rdst" -lt "128" ]]; then # if no input, do nothing, just print line on the screen
             inline="$inline$char" # append input character to the line
             INBUFFER=$inline # write line to fifo buffer
           fi
        fi
    fi
  done
# Writing input line to chat archive files
    line=$INBUFFER
    echo
    if [ "${line::1}" == ":" ]
    then
        echo $MEMBER 'runs bash command `'"${line:1}"'`' | ./suc.sh ".$CHANNEL" "$MEMBER" "$PASSWORD";
        bash -c "${line:1}" | ./suc.sh "$CHANNEL" "$MEMBER" "$PASSWORD";
    else
        if [ "$line" == "~DESTROY~" ] # user request to destroy chat archive
        then
            echo "!!! DESTROYING \"$CHANNEL\" CHAT ARCHIVE!!!" > ./usuc.msg
            printf "\033[A\33[2K\r" # move cursor up and clear the line
            echo
            kill $tailpidout; # kill background tail process
            exit 0;
        fi
        if [ "$line" == "~CONNECT~" ] # user request to connect to remote host
        then
            echo "Connecting to Remote Host" >  ./usuc.msg;
            printf "\033[A\33[2K\r" # move cursor up and clear the line
            echo
            kill $tailpidout; # kill background tail process
            exit 0;
        fi
        if [ "$line" == "~HISTORY~" ] # user request to connect to remote host
        then
            echo "Archived History of \"$CHANNEL\" Chat archive" >  ./usuc.msg;
            printf "\033[A\33[2K\r" # move cursor up and clear the line
            echo
            kill $tailpidout; # kill background tail process
            exit 0;
        fi
        printf "\033[A\33[2K\r" # move cursor up and clear the line
        echo "$line" | fmt | ./suc.sh "$CHANNEL" "$MEMBER" "$PASSWORD" # carry writings to chat out archive file
    fi
done

# End
# ------------------------------------------

exit 1





